
#ifndef __DC_LIB_DEF_H_____
#define __DC_LIB_DEF_H_____

#ifdef __cplusplus
extern "C" {
#endif

#define		CAP_BASE_ERROR_START	0x0100
#define		CAP_CHAN_ERROR_START	0x0200
#define		CAP_CARD_ERROR_START	0x0300
#define		CAP_LIB_ERROR_START		0x0400

// error code

enum {
	CAP_BASE_DEVICE_OPEN_ERROR	= CAP_BASE_ERROR_START,
	CAP_BASE_NO_HANDLE,
	CAP_CARD_CHANNEL_OVERFLOW	= CAP_CARD_ERROR_START,
	DC_ERR_NO_DEVICE			= CAP_LIB_ERROR_START,
	DC_ERR_NO_HANDLE,
	DC_ERR_NOT_INITIALIZED,
	DC_ERR_CHANNEL_INVALID,

	DC_ERR_ALREADY_OPENED,
	DC_ERR_OPEN_BITS_PER_CLK_INVALID,
	DC_ERR_POSITION_INVALID,
	DC_ERR_FRAMES_INVALID,
	DC_ERR_FRAME_SIZE_INVALID,
	DC_ERR_FRAME_BUFFER_ALLOC_FAILS,
	DC_ERR_START_FAILS,
	DC_ERR_ALREADY_PENDING,
	DC_ERR_NOT_PENDING,
	DC_ERR_CARD_NUMBER_INVALID,
	DC_ERR_CHANNEL_NOT_OPENED,
	DC_ERR_ALREADY_STARTED,
	DC_ERR_NOT_STARTED,
	DC_ERR_BUFFER_OVERFLOW, 
	DC_ERR_FRAME_OVER_RUN, 
	DC_ERR_DMA_TRANSFER_FAILS,
	DC_ERR_NO_GET_FRAME_PENDING,
	DC_ERR_LUT_INDEX_RANGE_INVALID
};


#define	BYTE	unsigned char
#define WORD	unsigned short
#define DWORD	unsigned long


__declspec(dllexport ) int DC_Init(int *handle);
__declspec(dllexport ) int DC_End(int handle);

__declspec(dllexport ) int DC_GetMaxCards(int handle);
__declspec(dllexport ) int DC_SelectCard(int handle,int card_num);
__declspec(dllexport ) int DC_GetCurCard(int handle);

__declspec(dllexport ) int DC_GetVersion(int handle,DWORD *DllVer,DWORD *DrvVer,DWORD *ChipVer);

__declspec(dllexport ) int DC_GetMaxChannels(int handle);
__declspec(dllexport ) int DC_WriteReg(int handle,DWORD offset,DWORD data);
__declspec(dllexport ) int DC_ReadReg(int handle,DWORD offset,DWORD *data);

__declspec(dllexport ) int DC_GenIoIrqSet(int handle,DWORD ipen, DWORD inen);
__declspec(dllexport ) int DC_GenIoRead(int handle,DWORD *data);
__declspec(dllexport ) int DC_GenIoReadOvl(int handle,DWORD *data,DWORD *status);
__declspec(dllexport ) int DC_GenIoCancelReadOvl(int handle);
__declspec(dllexport ) int DC_GenIoWrite(int handle,DWORD data);

__declspec(dllexport ) int DC_GpioReadIn(int handle,DWORD *data);
__declspec(dllexport ) int DC_GpioWriteOut(int handle,DWORD data);
__declspec(dllexport ) int DC_GpioWriteOE(int handle,DWORD data);

__declspec(dllexport ) int DC_Open(int handle,int channel,int Height,int Width,int Frames,int BitsPerClk);
__declspec(dllexport ) int DC_Close(int handle,int channel);

__declspec(dllexport ) int DC_Start(int handle,int channel);
__declspec(dllexport ) int DC_Stop(int handle,int channel);



__declspec(dllexport ) int DC_GetFrame(int handle,int channel, int no_blocked,BYTE **buf,WORD *length);
__declspec(dllexport ) int DC_CancelGetFrame(int handle,int channel);

__declspec(dllexport ) int DC_SetCameraCableMode(int handle,int channel,int mode);
__declspec(dllexport ) int DC_SetCapInfo(int handle,int channel,int HBP, int VBP, int ClkEdgeHigh, int HSyncEdgeHigh, int VSyncEdgeHigh, int LineArea, int Input10Bits);
__declspec(dllexport ) int DC_SetFilter(int handle,int channel,DWORD mode,DWORD fsHz,DWORD cutoffHz,int offset,float q);

__declspec(dllexport ) int DC_SetExposure(int handle,int channel,int enable,int ActiveHigh,WORD width);
__declspec(dllexport ) int DC_SetLookupTable(int handle,int channel,int index, WORD value);

// encoder 
__declspec(dllexport ) int DC_SetEncCounter(int handle,int channel,DWORD val);
__declspec(dllexport ) int DC_GetEncCounter(int handle,int channel,DWORD *val);

__declspec(dllexport ) int DC_SetTrgH(int handle,int channel,int CountSrc,int CntSrcInvert,int TrgSrc,int TrgSrcInvert,int OutInvert,WORD Width,WORD Numbers);
__declspec(dllexport ) int DC_SetTrgV(int handle,int channel,int CountSrc,int CntSrcInvert,int TrgSrc,int TrgSrcInvert,int OutInvert,WORD Width,WORD Numbers);
__declspec(dllexport ) int DC_SetSyncOut(int handle,int channel,int Source,int Invert,int OE);
__declspec(dllexport ) int DC_SyncOutToggle(int handle,int channel,WORD width);

// PLL control functions
__declspec(dllexport ) int DC_SetClkOutFreq(int handle,int clk_o_number, int ref_number);

// I2C control functions
__declspec(dllexport ) void DC_I2cInit(int handle,int channel,int DelayCnt);
__declspec(dllexport ) void DC_I2cRst(int handle,int channel);
__declspec(dllexport ) void DC_I2cWrite(int handle,int channel,BYTE data);
__declspec(dllexport ) int DC_I2cRead(int handle,int channel);
__declspec(dllexport ) void DC_I2cStart(int handle,int channel);
__declspec(dllexport ) void DC_I2cStop(int handle,int channel);

// EEPROM program
__declspec(dllexport ) int DC_Prog(int handle,void *report, char *filename);

#ifdef __cplusplus
}
#endif

#endif
