#include <stdio.h>
#include "StdAfx.h"
#include "imglib.h"


BOOL ReadRAWImage(LPCTSTR pRawFile, BYTE *chImage, WORD Width, WORD Height)
{
	FILE *fp;

	if((fp = fopen(pRawFile, "rb")) == NULL) return FALSE;
	if(fread(chImage, sizeof(BYTE), (Width * Height), fp) != (unsigned)(Width * Height)){
		fclose(fp);
		return FALSE;
	}
	fclose(fp);

	return TRUE;
}

BOOL WriteRAWImage(LPCTSTR pRawFile, BYTE *chImage, WORD Width, WORD Height)
{
	FILE *fp;

	if((fp = fopen(pRawFile, "wb")) == NULL) return FALSE;
	if(fwrite(chImage, sizeof(BYTE), (Width * Height), fp) != (unsigned)(Width * Height)){
		fclose(fp);
		return FALSE;
	}
	fclose(fp);

	return TRUE;
}

BOOL ReadBMPImage(LPCTSTR pBmpFile, BYTE *chImage, WORD Width, WORD Height)
{
	FILE* fp;
	BITMAPFILEHEADER bmfHdr;
	BITMAPINFOHEADER bmiHdr;

	if((fp = fopen(pBmpFile, "rb")) == NULL) return FALSE;

	if(fread(&bmfHdr, sizeof(BYTE), sizeof(BITMAPFILEHEADER), fp) != sizeof(BITMAPFILEHEADER)) return FALSE;
	if(bmfHdr.bfType != ((WORD)'M'<<8 | 'B')) return FALSE;
	if(fread(&bmiHdr, sizeof(BYTE), sizeof(BITMAPINFOHEADER), fp) != sizeof(BITMAPINFOHEADER)) return FALSE;
	if(bmiHdr.biBitCount != 24) return FALSE;

	if(fread(chImage, sizeof(BYTE), (Width * Height * 3), fp) != (unsigned)(Width * Height * 3)) return FALSE;

	fclose(fp);

	return TRUE;
}

BOOL WriteBMPImage(LPCTSTR pBmpFile, BYTE *chImage, WORD Width, WORD Height)
{
	FILE* fp;
	BITMAPFILEHEADER bmfHdr;
	BITMAPINFOHEADER bmiHdr;
	
    MakeBMPHeader(&bmiHdr, &bmfHdr, Width, Height);

	if((fp = fopen(pBmpFile, "wb")) == NULL) return FALSE;

	if(fwrite(&bmfHdr, sizeof(BYTE), sizeof(BITMAPFILEHEADER), fp) != sizeof(BITMAPFILEHEADER)) return FALSE;
	if(fwrite(&bmiHdr, sizeof(BYTE), sizeof(BITMAPINFOHEADER), fp) != sizeof(BITMAPINFOHEADER)) return FALSE;

	if(fwrite(chImage, sizeof(BYTE), (Width * Height * 3), fp) != (unsigned)(Width * Height * 3)) return FALSE;

	fclose(fp);
	
	return TRUE;
}

void MakeBMPHeader(BITMAPINFOHEADER *pbmiHdr, BITMAPFILEHEADER *pbmfHdr, WORD Width, WORD Height)
{
	memset(pbmfHdr, 0, sizeof(BITMAPFILEHEADER));
	pbmfHdr->bfType = ((WORD)'M'<<8 | 'B');
	pbmfHdr->bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + (Width * Height * 3);
	pbmfHdr->bfReserved1 = 0;
	pbmfHdr->bfReserved2 = 0;
	pbmfHdr->bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);

	memset(pbmiHdr, 0, sizeof(BITMAPINFOHEADER));
	pbmiHdr->biSize = sizeof(BITMAPINFOHEADER);
	pbmiHdr->biWidth = Width;
	pbmiHdr->biHeight = Height;
	pbmiHdr->biPlanes = 1;
	pbmiHdr->biBitCount = 24;
	pbmiHdr->biCompression = BI_RGB;
	pbmiHdr->biSizeImage = (Width * Height * 3);
}


void RAWtoBMPImage(BYTE *rawImage, BYTE *bmpImage, WORD Width, WORD Height)
{
	BYTE data;

	for(int i = 0;  i < Height; i++) {
		for(int j = 0; j < Width; j++) {
			data = rawImage[Width * i + j];

			//if (data & 0x80) data &= ~(1<<7);
			//else data |= (1<<7);

			//if (data & (1<<1)) data &= ~(1<<1);
			//else data |= (1<<1);

			//if (data & (1<<2)) data &= ~(1<<2);
			//else data |= (1<<2);

			//if (data & (1<<0)) data &= ~(1<<0);
			//else data |= (1<<0);

			//int k=0;
			//if (data & (1<<k)) data &= ~(1<<k);
			//else data |= (1<<k);

			//data |= (1<<0);
			//data |= (1<<1);
			//data |= (1<<2);
			//data |= (1<<3);
			//data |= (1<<4);
			//data |= (1<<5);
			//data |= (1<<6);
			//data |= (1<<7);

			bmpImage[((Width * (Height - (i + 1)) + j )* 3)] = data;
			bmpImage[((Width * (Height - (i + 1)) + j )* 3)+1] = data;
			bmpImage[((Width * (Height - (i + 1)) + j )* 3)+2] = data;
		}
	}
}


void RAWtoBMPImageRatio(BYTE ratio,BYTE *rawImage, BYTE *bmpImage, WORD Width, WORD Height)
{
	for(int i = 0;  i < Height; i++) {
		for(int j = 0; j < Width; j++) {
			bmpImage[((Width * (Height - (i + 1)) + j )* 3)] = ~rawImage[Width*ratio * i + j*ratio];
			bmpImage[((Width * (Height - (i + 1)) + j )* 3) + 1] = ~rawImage[Width*ratio * i + j*ratio];
			bmpImage[((Width * (Height - (i + 1)) + j )* 3) + 2] = ~rawImage[Width*ratio * i + j*ratio];
		}
	}
}



void BMPtoRAWImage(BYTE *bmpImage, BYTE *rawImage, WORD Width, WORD Height)
{
	for(int i = 0;  i < Height; i++) {
		for(int j = 0; j < Width; j++) {
			rawImage[Width * i + j] = (BYTE)((float)(bmpImage[((Width * (Height - (i + 1)) + j )* 3)] 
				+ bmpImage[((Width * (Height - (i + 1)) + j )* 3) + 1]
				+ bmpImage[((Width * (Height - (i + 1)) + j )* 3) + 2]) / 3.f);
		}
	}
}


void YUVtoBMPImage(BYTE *rawImage,BYTE *bmpImage,int Width,int Height)
{
	int		r1, g1, b1, r2, g2, b2, u, v, cb, cr,y1,y2;
	int     i,j;
	int     xxx,yyy;
	int     yuvcount;
	int     rgbcount;
	 
	yuvcount = 0;

    for( yyy = Height-1 ; yyy >= 0 ; yyy--) {
		rgbcount =   (yyy*Width*3);
        for( xxx = 0 ; xxx < Width ; xxx++) {

			cb = rawImage[ yuvcount*4 + 2] - 128;
            y1 = rawImage[ yuvcount*4 + 1];
			cr = rawImage[ yuvcount*4 + 0] - 128;
            y2 = rawImage[ yuvcount*4 + 3];

			yuvcount++;
			/////////////////////////////////	
			u = cb * 2 + 1;
			v = (cr * 1600 + 800)/1000;

			////////////////////////////////////
			r1 = v + y1;
			b1 = u + y1;
			//g1 = (int)((y1 - 0.299 * r1 - 0.114 * b1) / 0.587);       
			g1 = (int)((1000*y1 - 299 * r1 - 114 * b1) / 587);       

			if(r1 <= 0 ) r1 = 0;  if(r1 > 255) r1 = 255;
			if(g1 <= 0)  g1 = 0;  if(g1 > 255) g1 = 255;
			if(b1 <= 0)  b1 = 0;  if(b1 > 255) b1 = 255;

            bmpImage[rgbcount+0] = (BYTE)r1;
            bmpImage[rgbcount+1] = (BYTE)g1;
            bmpImage[rgbcount+2] = (BYTE)b1;


			xxx++;  // xdot ϳ ...
           	rgbcount += 3;

			r2 = v + y2;
			b2 = u + y2;
			g2 = (int)((1000*y2 - 299 * r2 - 114 * b2) / 587);

			if(r2 <= 0 ) r2 = 0;  if(r2 > 255) r2 = 255;
			if(g2<= 0)  g2 = 0;  if(g2 > 255) g2 = 255;
			if(b2<= 0)  b2 = 0;  if(b2 > 255) b2 = 255;	

            bmpImage[rgbcount+0] = (BYTE)r2;
            bmpImage[rgbcount+1] = (BYTE)g2;
            bmpImage[rgbcount+2] = (BYTE)b2;
           	rgbcount += 3;
		}
	}
}


