// Chan.cpp: implementation of the CChan class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "dclibdef.h"
#include "View.h"
#include "Chan.h"
#include "imglib.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CChan::CChan(int ch,CFormView *pView,CDclibWrap*cap)
{
	m_Cap = cap;
	m_Ch = ch;
	m_X = 116;
	m_Y = 4;
	m_Frames = 3;
	m_FPS = 0;
	m_SlAddr = 0xba;
	m_bRunning = 0;
	m_CapMode = 0;
	m_pView = pView;
}


CChan::~CChan()
{
}


/////////////////////////////////////////////
// capture channel related parameters
/////////////////////////////////////////////

void CChan::CamReset() 
{
	// TODO: Add your control notification handler code here
	m_Cap->I2cRst(m_Ch);
	SetCapMode(m_CapMode);
}


void CChan::I2cWrite(int addr,UINT data)
{
	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,m_SlAddr);
	m_Cap->I2cWrite(m_Ch,addr&0xff);
	m_Cap->I2cWrite(m_Ch,(data>>8));		// high

	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,m_SlAddr);
	m_Cap->I2cWrite(m_Ch,0xF1);
	
	m_Cap->I2cWrite(m_Ch,data & 0xff);	// low
	m_Cap->I2cStop(m_Ch);
}


UINT CChan::I2cRead(int addr)
{
	UINT data;

	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,m_SlAddr);
	m_Cap->I2cWrite(m_Ch,addr&0xff);

	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,m_SlAddr+1);
	data = m_Cap->I2cRead(m_Ch);

	data = data<<8;
	data &= 0xff00;

	
	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,m_SlAddr);
	m_Cap->I2cWrite(m_Ch,0xF1);

	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,m_SlAddr+1);
	
	data += m_Cap->I2cRead(m_Ch);
	m_Cap->I2cStop(m_Ch);
	return data;
}


void CChan::SetCapMode(int mode)
{
	m_CapMode = mode;

	m_Height = 512;
	m_Width = 640;

	if (m_CapMode==1) {
		m_Height = m_Height*2;
		m_Width = m_Width*2;
	}

	I2cWrite(0xf0,2);
	if (m_CapMode==1) {
		// 1280x1024 mode
		I2cWrite(0xc8,0x9f0b);
	}
	else {
		I2cWrite(0xc8,0);
	}
	I2cWrite(0xf0,0);

	CRect rectImage(m_X,m_Y,m_Width+m_X,m_Height+m_Y);
	m_Rect = rectImage;
}


void CChan::SetDispMode(int mode)
{
	CRect rectImage(m_X,m_Y,m_Width+m_X,m_Height+m_Y);
	m_DispMode = mode;
	m_Rect = rectImage;

}



UINT ThreadFunc(LPVOID lpData){
	CChan *pChan = (CChan *)lpData;
	return pChan->Capture();
}

void CChan::Start()
{
	m_bRunning = TRUE;
	AfxBeginThread(ThreadFunc, (LPVOID)(this), THREAD_PRIORITY_NORMAL);
}


void CChan::Stop()
{
	m_bRunning = FALSE;
	m_Cap->CancelGetFrame(m_Ch);
}


UINT CChan::Capture()
{
	m_Cap->Open(m_Ch,m_Height,m_Width*2,m_Frames,8);	// YUV mode
	m_Cap->SetCapInfo(	
					m_Ch,
					0,	// HBP
					0,	// VBP
					1,	// ClkEdgeHigh
					1,	// HSyncEdgeHigh
					1,	// VSyncEdgeHigh
					0,	// LineArea
					0	// Input10Bits
					);

	m_Cap->Start(m_Ch);

	while(m_bRunning){
		BYTE *buf;
		WORD length;

		m_Cap->GetFrame(m_Ch,0,&buf,&length);
		if (length>0 && buf)  {
			if (m_DispMode) {
				YUVtoBMPImage((BYTE *)buf, m_BmpImage,m_Width, m_Height);
				m_pView->InvalidateRect(&m_Rect, FALSE);
			}
			m_FPS++;
		}
	}
	m_Cap->Stop(m_Ch);
	m_Cap->Close(m_Ch);
	return 0;
}



void CChan::Draw(CPaintDC *dc)
{
	if (m_bRunning) {
		if (m_DispMode==1) {
			BITMAPINFOHEADER m_bmiHdr;
			BITMAPFILEHEADER m_bmfHdr;

			MakeBMPHeader(&m_bmiHdr, &m_bmfHdr, m_Width, m_Height);

			SetDIBitsToDevice(dc->m_hDC, m_X, m_Y,	m_bmiHdr.biWidth, m_bmiHdr.biHeight,
								0, 0, 0, (WORD)m_bmiHdr.biHeight, m_BmpImage,
								(LPBITMAPINFO)&m_bmiHdr, BI_RGB);
		}
	}
}



// call this function every one second
int CChan::GetFPS()
{
	int ret;
	ret = m_FPS;
	m_FPS = 0;
	return ret;
}


BOOL CChan::IsRunning()
{
	return m_bRunning;
}


void CChan::SaveImage()
{
	char name[255];

	time_t osBinaryTime;  // C run-time time (defined in <time.h>)
	time( &osBinaryTime ) ;  // Get the current time from the 
							 // operating system.
	CTime time3(osBinaryTime );  

	sprintf(name,"img%1d_%4d%02d%02d%02d%02d%02d.bmp",m_Ch,time3.GetYear(),time3.GetMonth(),time3.GetDay(),time3.GetHour(),time3.GetMinute(),time3.GetSecond());

	WriteBMPImage(name, m_BmpImage,m_Width,m_Height);
}
